#! /usr/bin/python
#***********************************************************
#* Software License Agreement (BSD License)
#*
#*  Copyright (c) 2009, Willow Garage, Inc.
#*  All rights reserved.
#*
#*  Redistribution and use in source and binary forms, with or without
#*  modification, are permitted provided that the following conditions
#*  are met:
#*
#*   * Redistributions of source code must retain the above copyright
#*     notice, this list of conditions and the following disclaimer.
#*   * Redistributions in binary form must reproduce the above
#*     copyright notice, this list of conditions and the following
#*     disclaimer in the documentation and/or other materials provided
#*     with the distribution.
#*   * Neither the name of Willow Garage, Inc. nor the names of its
#*     contributors may be used to endorse or promote products derived
#*     from this software without specific prior written permission.
#*
#*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#*  POSSIBILITY OF SUCH DAMAGE.
#* 
#* Author: Eitan Marder-Eppstein
#***********************************************************
PKG = 'continuous_ops_web'
NAME = 'task_list_server'
import time
import roslib; roslib.load_manifest(PKG)

from continuous_ops_msgs.srv import *
from continuous_ops_msgs.msg import *
import rospy

tasks = {}

def get_tasks(req):
    print "getting task list"
    resp = GetTaskListResponse()
    for i in range(10):
        task = Task()
        task.name = 'task%d' % i
        task.task_id = '%d' % i
        task.image_file = '/wg/bwi/eitan/ros/dev/continuous_ops/continuous_ops_web/src/continuous_ops_web/images/cont.jpg'
        resp.tasks.append(task)
    available_pub.publish(resp.tasks)
    return resp

def add_task(req):
    print "adding a task: %s" % req.task_id
    time.sleep(1)
    print "done sleeping"
    tasks['run%s' % req.task_id] = req.task_id
    resp = AddTaskResponse()
    resp.run_id = 'run%s' % req.task_id
    for key in tasks.keys():
        task = Task()
        task.name = 'task%s' % tasks[key]
        task.task_id = tasks[key]
        task.run_id = key
        resp.tasks.append(task)
    print resp
    return resp

def remove_task(req):
    print "removing task: %s" % req.run_id
    time.sleep(1)
    del tasks[req.run_id]
    resp = ChangeTaskResponse()
    for key in tasks.keys():
        task = Task()
        task.name = 'task%s' % tasks[key]
        task.task_id = tasks[key]
        task.run_id = key
        resp.tasks.append(task)
    print resp
    return resp

def get_scheduled_tasks(req):
    print "serving scheduled tasks"
    resp = GetTaskListResponse()
    for key in tasks.keys():
        task = Task()
        task.name = 'task%s' % tasks[key]
        task.task_id = tasks[key]
        task.run_id = key
        resp.tasks.append(task)
    print resp
    return resp

def reorder_tasks(req):
    print "reorder"
    time.sleep(1)
    print req
    resp = ReorderTaskListResponse()
    for key in tasks.keys():
        task = Task()
        task.name = 'task%s' % tasks[key]
        task.task_id = tasks[key]
        task.run_id = key
        resp.tasks.append(task)
    print resp
    return resp

def task_list_server():
    rospy.init_node(NAME)
    get = rospy.Service('get_available_tasks', GetTaskList, get_tasks)
    get_s = rospy.Service('get_scheduled_tasks', GetTaskList, get_scheduled_tasks)
    reorder = rospy.Service('reorder_tasks', ReorderTaskList, reorder_tasks)
    add = rospy.Service('add_task', AddTask, add_task)
    remove = rospy.Service('remove_task', ChangeTask, remove_task)

    global available_pub
    available_pub = rospy.Publisher('available_tasks', Tasks, None, False, True)
    rospy.spin()

if __name__ == "__main__":
    task_list_server()
